from typing import Dict

from change_case import ChangeCase
from jsonschema import ValidationError

from jsonschema2ddl.types import COLUMNS_TYPES_PREFERENCE


def db_table_name(table_name: str, schema_name: str = None) -> str:
    table_name = ChangeCase.camel_to_snake(table_name)
    if schema_name is None:
        return f'"{table_name}"'
    else:
        return f'"{schema_name}"."{table_name}"'


def db_column_name(col_name: str) -> str:
    return ChangeCase.camel_to_snake(col_name)


def get_one_schema(object_schema) -> Dict:
    """Get one schema from a list of 'allOf', 'anyOf', 'oneOf'. """
    types = object_schema.get('allOf', object_schema.get('anyOf', object_schema.get('oneOf', [])))
    if not types:
        raise ValidationError("Neither 'type', 'allOf', 'anyOf', 'oneOf' defined for field. Aborting!")
    types_with_preference = map(lambda obj: (obj, COLUMNS_TYPES_PREFERENCE[obj['type']]), types)
    max_preference_type = max(types_with_preference, key=lambda x: x[1])[0]
    return max_preference_type



from enum import Enum


# TODO: Define the types as enum
class Type(Enum):
    pass


POSTGRES_TYPES = {
    'boolean': 'bool',
    'number': 'float',
    'string': 'varchar({})',
    'enum': 'text',
    'integer': 'bigint',
    'timestamp': 'timestamptz',
    'date': 'date',
    'link': 'integer',
    'array': 'json',
    'object': 'json',
    'id': 'serial',
}

REDSHIFT_TYPES = {
    **POSTGRES_TYPES,
    'array': 'varchar(1024)',
    'object': 'varchar(1024)',
    'id': 'int identity(1, 1) not null',
}

COLUMNS_TYPES = {
    'postgres': POSTGRES_TYPES,
    'redshift': REDSHIFT_TYPES,
}

FK_TYPES = {
    'serial': 'bigint',
    'int identity(1, 1) not null': 'bigint',
}

COLUMNS_TYPES_PREFERENCE = {
    'boolean': 0,
    'bool': 0,
    'enum': 1,
    'link': 2,
    'integer': 3,
    'bigint': 4,
    'number': 5,
    'float': 5,
    'date': 5,
    'timestamp': 6,
    'timestamptz': 6,
    'array': 7,
    'text': 8,
    'string': 8,
    'object': 9,
    'json': 9,
    'id': 10,
    'serial': 10,
}
